package foo.payment.plugin;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import org.slf4j.Logger;
import ru.crystals.pos.api.ext.loyal.dto.Slip;
import ru.crystals.pos.api.ext.loyal.dto.SlipParagraph;
import ru.crystals.pos.api.ext.loyal.dto.SlipParagraphType;
import ru.crystals.pos.api.plugin.FunctionPlugin;
import ru.crystals.pos.api.plugin.function.CreateReceiptResult;
import ru.crystals.pos.api.plugin.function.FunctionEnableResult;
import ru.crystals.pos.api.plugin.function.VoidResult;
import ru.crystals.pos.api.plugin.function.dto.Payment;
import ru.crystals.pos.api.plugin.function.dto.PaymentType;
import ru.crystals.pos.api.plugin.function.dto.Position;
import ru.crystals.pos.api.plugin.function.dto.PositionDataKey;
import ru.crystals.pos.api.plugin.function.dto.Product;
import ru.crystals.pos.api.ui.listener.InputListener;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.POSInfo;
import ru.crystals.pos.spi.ResBundle;
import ru.crystals.pos.spi.annotation.Inject;
import ru.crystals.pos.spi.annotation.POSPlugin;
import ru.crystals.pos.spi.plugin.function.FunctionRequest;
import ru.crystals.pos.spi.receipt.Receipt;
import ru.crystals.pos.spi.receipt.ReceiptType;
import ru.crystals.pos.spi.ui.UIForms;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Stream;

@POSPlugin(id = "foo.service.function.create.receipt")
public class CreateReceiptFunctionPlugin implements FunctionPlugin {

    enum ItemKey {
        SALE,
        SALE_MARKING,

        REFUND,

        REFUND_RECEIPT_IS_EDITABLE_FALSE,

        REFUND_POSITION_IS_EDITABLE_FALSE,

        REFUND_WITH_CONTROL_ORDER_PAYMENTS,

        REFUND_ITEM_NOT_EXIST,

        REFUND_MARKING
    }

    private static final String ITEM_NOT_EXIST = "01";

    private static final List<String> ITEMS = Arrays.asList(
            "00001",
            "00005",
            "5029053534572",
            "79025",
            "15004",
            "011361",
            "79026"
    );

    private List<String> MARKING_ITEMS = Arrays.asList(
            "99998"
    );

    @Inject
    private POSInfo pos; // параметры кассы
    
    @Inject
    private IntegrationProperties properties; // настройки интеграции и плагина

    @Inject
    private UIForms ui; // формы кассы

    @Inject
    private Logger log; // логирование
    
    @Inject
    private ResBundle res; // локализованные ресурсы

    @PostConstruct
    private void init() {
        log.info("Plugin {} loaded", this.getClass());
    }

    @Override
    public void execute(FunctionRequest request) {
        log.info("Call method execute");
        Map<String, List<String>> items = new LinkedHashMap<>();
        items.put(ItemKey.SALE.name(), Collections.singletonList("Продажа"));
        items.put(ItemKey.SALE_MARKING.name(), Collections.singletonList("Продажа (маркировка)"));
        items.put(ItemKey.REFUND.name(), Collections.singletonList("Возврат"));
        items.put(ItemKey.REFUND_WITH_CONTROL_ORDER_PAYMENTS.name(), Collections.singletonList("Возврат с контролем порядка оплат"));
        items.put(ItemKey.REFUND_ITEM_NOT_EXIST.name(), Collections.singletonList("Возврат несуществующего товара"));
        items.put(ItemKey.REFUND_MARKING.name(), Collections.singletonList("Возврат (маркировка)"));
        items.put(ItemKey.REFUND_RECEIPT_IS_EDITABLE_FALSE.name(), Collections.singletonList("Возврат (чек с запретом редактирования)"));
        items.put(ItemKey.REFUND_POSITION_IS_EDITABLE_FALSE.name(), Collections.singletonList("Возврат (первая позиция с запретом редактирования)"));

        ui.getInputForms().showSelectionForm("Тип документа", items, new InputListener() {
            @Override
            public void eventInputComplete(String value) {
                switch (ItemKey.valueOf(value)) {
                    case SALE:
                        createSaleReceipt(request, false, false);
                        break;
                    case SALE_MARKING:
                        createSaleReceipt(request, true, false);
                        break;
                    case REFUND:
                        createRefundReceipt(request, false, false);
                        break;
                    case REFUND_RECEIPT_IS_EDITABLE_FALSE:
                        createRefundReceiptIsEditableFalse(request);
                        break;
                    case REFUND_POSITION_IS_EDITABLE_FALSE:
                        createRefundReceiptIsFirstPositionEditableFalse(request);
                        break;
                    case REFUND_WITH_CONTROL_ORDER_PAYMENTS:
                        createRefundReceipt(request, false, true);
                        break;
                    case REFUND_ITEM_NOT_EXIST:
                        createRefundReceiptItemNotExist(request);
                        break;
                    case REFUND_MARKING:
                        createRefundReceipt(request, true, false);
                        break;
                    default:
                        request.getCallback().completed();
                }
            }

            @Override
            public void eventCanceled() {
                request.getCallback().completed();
            }
        });

    }

    private void createRefundReceiptItemNotExist(FunctionRequest request) {
        request.getCallback().completed(createReceiptResultItemNotExist(ReceiptType.REFUND));
    }

    private void createRefundReceipt(FunctionRequest request, boolean marking, boolean controlOrderPayments) {
        request.getCallback().completed(createReceiptResult(ReceiptType.REFUND, marking, controlOrderPayments));
    }

    private void createRefundReceiptIsEditableFalse(FunctionRequest request) {
        CreateReceiptResult result = createReceiptResult(ReceiptType.REFUND, false, false);
        result.getReceipt().setEditable(false);
        request.getCallback().completed(result);
    }

    private void createRefundReceiptIsFirstPositionEditableFalse(FunctionRequest request) {
        CreateReceiptResult result = createReceiptResult(ReceiptType.REFUND, false, false);
        result.getReceipt().getPositions().get(0).setEditable(false);
        request.getCallback().completed(result);
    }

    private void createSaleReceipt(FunctionRequest request, boolean marking, boolean controlOrderPayments) {
        request.getCallback().completed(createReceiptResult(ReceiptType.SALE, marking, controlOrderPayments));
    }

    @Override
    public FunctionEnableResult enabled(Optional<Receipt> receipt) {
        return receipt.isPresent() ? new FunctionEnableResult(false, res.getString("search.enabled.message")) : new FunctionEnableResult();
    }

    private CreateReceiptResult createReceiptResultItemNotExist(ReceiptType type) {
        ru.crystals.pos.api.plugin.function.dto.Receipt receipt = new ru.crystals.pos.api.plugin.function.dto.Receipt(type);

        Position position = createPosition(1, ITEM_NOT_EXIST, 1000L);
        receipt.getPositions().add(position);

        receipt.getPayments().add(createPayment(PaymentType.CASH.getTypeId(), position.getAmount()));

        receipt.setControlOrderPayments(true);
        receipt.setSlips(createSlips());
        receipt.setEditable(true);

        return new CreateReceiptResult(receipt);
    }

    private CreateReceiptResult createReceiptResult(ReceiptType type, boolean marking, boolean controlOrderPayments) {
        ru.crystals.pos.api.plugin.function.dto.Receipt receipt = new ru.crystals.pos.api.plugin.function.dto.Receipt(type);

        receipt.getPositions().addAll(createPositions(marking));

        receipt.getPayments().addAll(createPayments(receipt.getPositions()));

        receipt.setControlOrderPayments(controlOrderPayments);
        receipt.setSlips(createSlips());
        receipt.setEditable(true);

        return new CreateReceiptResult(receipt);
    }

    private List<Position> createPositions(boolean marking) {
        List<Position> positions = new ArrayList<>();
        final int[] positionNumber = {1};

        ITEMS.forEach(i -> {
            positions.add(createPosition(positionNumber[0], i, 1000L));
            positionNumber[0]++;
        });

        if (marking) {
            MARKING_ITEMS.forEach(i -> {
                positions.add(createPosition(positionNumber[0], i, 1000L));
                positionNumber[0]++;
            });
        }

        return positions;
    }

    private Position createPosition(int number, String item, Long quantity) {
        Product product = new Product();
        product.setItem(item);
        product.setBarcode(item);
        product.setName("Товар: " + item);
        product.setPrice(BigDecimal.valueOf(8.54));
        product.setVatRate(20F);

        Position position = new Position(product);
        position.setNumber(number);
        position.setQuantity(quantity);
        position.setAmount(product.getPrice().multiply(BigDecimal.valueOf(quantity, 3)));
        position.getData().put("TEST_KEY", "test value");
        position.getData().put(PositionDataKey.REFUND_REASON.name(), "reason 1");
        position.setEditable(true);

        return position;
    }

    private List<Payment> createPayments(List<Position> positions) {
        List<Payment> payments = new ArrayList<>();
        BigDecimal receiptAmount = positions.stream().map(Position::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal onePaymentAmount = receiptAmount.divide(BigDecimal.valueOf(6), 2, RoundingMode.CEILING);

        Stream.of(PaymentType.values()).forEach(
                pt -> payments.add(createPayment(pt.getTypeId(), onePaymentAmount))
        );

        payments.add(createPayment("foo.service.payment", onePaymentAmount));
        payments.add(createPayment("foo.service.payment", onePaymentAmount));

        return payments;
    }

    private Payment createPayment(String type, BigDecimal amount) {
        Payment payment = new Payment(type, amount);
        return payment;
    }

    private List<Slip> createSlips() {
        Slip slip = new Slip();
        slip.getParagraphs().add(new SlipParagraph(SlipParagraphType.TEXT, "Тестовый слип"));
        slip.getParagraphs().add(new SlipParagraph(SlipParagraphType.BARCODE, "01234567890"));

        return Arrays.asList(slip);
    }
 }
