package ru.crystals.pos.api.plugin.card;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import ru.crystals.pos.api.card.CardEntity;
import ru.crystals.pos.api.comm.CommunicationMessage;

/**
 * Сущность, описывающая отклик на запрос информации по карте/купону.
 */
public class CardSearchResponse {

    private CardSearchResponseStatus responseStatus;

    private CardEntity card;

    private Set<String> cardsToRemove;

    private List<String> cashierMessages = new ArrayList<>();

    /**
     * Список расширенных сообщений кассиру. В отличие от старых сообщений эти умеют управлять иконкой сообщения и
     * разделяют текст на семантические блоки.
     * @since 0.0.46
     */
    private List<CommunicationMessage> communicationMessages = new ArrayList<>();

    /**
     * Конструктор
     * 
     * @param responseStatus
     *            статус запроса карты
     * @see CardSearchResponseStatus
     */
    public CardSearchResponse(CardSearchResponseStatus responseStatus) {
        super();
        this.responseStatus = responseStatus;
    }

    /**
     * Конструктор
     * 
     * @param responseStatus
     *            статус запроса карты
     * @param card
     *            карта
     * @see CardSearchResponseStatus
     * @see CardEntity
     */
    public CardSearchResponse(CardSearchResponseStatus responseStatus, CardEntity card) {
        super();
        this.responseStatus = responseStatus;
        this.card = card;
    }

    /**
     * Получить статус отклика
     *
     * @return не {@code null}
     * @see CardSearchResponseStatus
     */
    public CardSearchResponseStatus getResponseStatus() {
        return responseStatus;
    }

    /**
     * Установить статус отклика
     * 
     * @param responseStatus статус
     * @see CardSearchResponseStatus
     */
    public void setResponseStatus(CardSearchResponseStatus responseStatus) {
        this.responseStatus = responseStatus;
    }

    /**
     * Вернет саму карту, запрос на получение информации по которой был сделан.
     *
     * @return {@code null}, если карта не найдена в справочнике карт по любой причине
     *         (т.е., если {@link #getResponseStatus() статус} отклика отличен от {@link CardSearchResponseStatus#OK})
     * @see CardEntity
     */
    public CardEntity getCard() {
        return card;
    }

    /**
     * Установить карту, запрос на получение информации по которой был сделан.
     * 
     * @param card
     *            {@code null}, если карта не найдена в справочнике карт по любой причине
     *            (т.е., если {@link #getResponseStatus() статус} отклика отличен от {@link CardSearchResponseStatus#OK})
     * @see CardEntity
     */
    public void setCard(CardEntity card) {
        this.card = card;
    }

    /**
     * Возвращает номера карт, которые должны быть удалены из чека (заменены найденной картой)
     * @return номера карт, которые должны быть удалены из чека
     * @since 0.0.46
     */
    public Set<String> getCardsToRemove() {
        if (cardsToRemove == null) {
            cardsToRemove = new HashSet<>();
        }
        return cardsToRemove;
    }

    /**
     * Список сообщений, что необходимо показать кассиру при добавлении данной карты в чек.
     * <p>
     * Например, попросить кассира напомнить покупателю о необходимости заполнить анкету или заменить карту.
     * Или вызвать охрану потому что карта в "черном" списке (например, украдена).
     * @return допускается возвращать {@code null} либо пустой список, если никаких сообщений кассиру показывать не требуется
     * @see #getCommunicationMessages()
     */
    public List<String> getCashierMessages() {
        return cashierMessages;
    }

    /**
     * Установить список сообщений, что необходимо показать кассиру при добавлении данной карты в чек.
     * @param cashierMessages сообщения
     * @see #getCashierMessages()
     * @deprecated в названии этого метода допущена синтаксическая ошибка (смешение кириллицы и латиницы),
     * посему метод объявлен как устаревший и не рекомендуется к использованию. Используйте метод {@link #addCashierMessages(List)}
     * вместо него.
     */
    @Deprecated
    public void addСashierMessages(List<String> cashierMessages) {
        this.addCashierMessages(cashierMessages);
    }

    /**
     * Добавить список сообщений кассиру.
     * @param messages список сообщений кассиру.
     * @see #getCashierMessages()
     */
    public void addCashierMessages(List<String> messages) {
        this.cashierMessages.addAll(messages);
    }
    /**
     * Добавляет сообщение кассиру.
     * @param cashierMessage текст сообщения кассиру.
     * @deprecated в названии этого метода допущена синтаксическая ошибка (смешение кириллицы и латиницы), посему
     * метод объявлен как устаревший и не рекомендуется к использованию. Используйте метод {@link #addCashierMessage(String)}
     * вместо него.
     */
    @Deprecated
    public void addСashierMessage(String cashierMessage) {
        addCashierMessage(cashierMessage);
    }

    /**
     * Добавляет сообщение кассиру.
     * @param message текст сообщения кассиру.
     * @see #getCashierMessages()
     */
    public void addCashierMessage(String message) {
        this.cashierMessages.add(message);
    }
    /**
     * Возвращают список "расширенных" сообщений кассиру, в которых допустимо управлять иконкой, а также делить
     * текст на смысловые блоки. Сюда можно добавить своих сообщений кассиру.
     * @return список расширенных сообщений кассиру, куда можно добавить своё сообщение, или пустой список, если таких нет.
     * @since 0.0.46
     * @see #getCashierMessages()
     */
    public List<CommunicationMessage> getCommunicationMessages() {
        return communicationMessages;
    }

    @Override
    public String toString() {
        return "CardSearchResponse{" +
                "responseStatus=" + responseStatus +
                ", card=" + card +
                ", cardsToRemove=" + cardsToRemove +
                ", cashierMessages=" + cashierMessages +
                ", communicationMessages=" + communicationMessages +
                '}';
    }
}
